package com.fjhx.xmhjc.service.column.impl;

import cn.hutool.core.util.StrUtil;
import com.fjhx.xmhjc.entity.column.dto.ColumnArticleSelectDto;
import com.fjhx.xmhjc.entity.column.po.ColumnMenuSub;
import com.fjhx.xmhjc.entity.column.vo.ColumnArticleVo;
import com.fjhx.xmhjc.mapper.column.ColumnMenuSubMapper;
import com.fjhx.xmhjc.service.column.ColumnArticleService;
import com.fjhx.xmhjc.service.column.ColumnMenuSubService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.xmhjc.entity.column.vo.ColumnMenuSubVo;
import com.fjhx.xmhjc.entity.column.dto.ColumnMenuSubSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.xmhjc.entity.column.dto.ColumnMenuSubDto;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 栏目-子栏目 服务实现类
 * </p>
 *
 * @author lqh
 * @since 2023-11-15
 */
@Service
public class ColumnMenuSubServiceImpl extends ServiceImpl<ColumnMenuSubMapper, ColumnMenuSub> implements ColumnMenuSubService {

    @Resource
    private ColumnArticleService columnArticleService;

    @Override
    public List<ColumnMenuSubVo> getList(ColumnMenuSubSelectDto dto) {
        IWrapper<ColumnMenuSub> wrapper = getWrapper();
        wrapper.eq("cms", ColumnMenuSub::getColumnId,dto.getColumnId());
        if(StrUtil.isNotBlank(dto.getStatus())){
            wrapper.eq("cms", ColumnMenuSub::getStatus, dto.getStatus());
        }
        wrapper.orderByAsc("cms", ColumnMenuSub::getSort);
        List<ColumnMenuSubVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<ColumnMenuSubVo> getPage(ColumnMenuSubSelectDto dto) {
        IWrapper<ColumnMenuSub> wrapper = getWrapper();
        if(StrUtil.isNotBlank(dto.getStatus())){
            wrapper.eq("cms", ColumnMenuSub::getStatus, dto.getStatus());
        }
        wrapper.orderByDesc("cms", ColumnMenuSub::getId);
        Page<ColumnMenuSubVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public ColumnMenuSubVo detail(Long id) {
        ColumnMenuSub ColumnMenuSub = this.getById(id);
        ColumnMenuSubVo result = BeanUtil.toBean(ColumnMenuSub, ColumnMenuSubVo.class);
        return result;
    }

    @Override
    public void add(ColumnMenuSubDto columnMenuSubDto) {
        this.save(columnMenuSubDto);
    }

    @Override
    public void edit(ColumnMenuSubDto columnMenuSubDto) {
        this.updateById(columnMenuSubDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public Boolean checkDeleteByMenu(Long id) {
        //查询子栏目
        ColumnMenuSubSelectDto columnMenuSubSelectDto = new ColumnMenuSubSelectDto();
        columnMenuSubSelectDto.setColumnId(id);
        List<ColumnMenuSubVo> list = getList(columnMenuSubSelectDto);
        //验证是否有文章
        long count = list.stream().filter(x -> {
            return !checkDelete(x.getId());
        }).count();
        if (count == 0){
            return true;
        }
        return false;
    }

    @Override
    public Boolean checkDelete(Long id) {
        ColumnArticleSelectDto columnArticleSelectDto = new ColumnArticleSelectDto();
        columnArticleSelectDto.setSubId(id);
        List<ColumnArticleVo> list = columnArticleService.getList(columnArticleSelectDto);
        if (list.isEmpty()){
            return  true;
        }
        return false;
    }

    @Transactional
    public void handleColumnMenuSub(Long id, List<ColumnMenuSub> columnMenuSubList) {
        //获取已有子栏目
        ColumnMenuSubSelectDto dto = new ColumnMenuSubSelectDto();
        dto.setColumnId(id);
        List<ColumnMenuSubVo> list = getList(dto);

        //用于删除子栏目
        Map<Long, ColumnMenuSubVo> map = list.stream().collect(Collectors.toMap(ColumnMenuSub::getId, Function.identity()));

        //新增修改
        columnMenuSubList.forEach(sub ->{
            sub.setColumnId(id);
            //判断新增修改
            if (Objects.isNull(sub.getId())){
               this.save(sub);
            }else {
                this.updateById(sub);
                //去除已存在Id
                map.remove(sub.getId());
            }
        });

        //删除
        map.forEach((key,value) ->{
            //TODO判断文章
            if (checkDelete(key)){
                this.removeById(key);
            }else {
                throw new RuntimeException(value.getName()+"存在文章，删除失败!如不想显示请禁用");
            }
        });

    }
}
